/*
 * HttpBCAsyncTester.java
 * 
 * Created on Aug 20, 2007, 12:19:35 PM
 * 
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

import java.io.FileInputStream;

import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.util.List;

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.Session;
import javax.jms.MessageConsumer;
import javax.jms.Message;
import com.sun.messaging.ConnectionFactory;
import com.stc.connector.httpadapter.http.HttpClientAPI;
import com.stc.connector.httpadapter.http.HttpResult;

/**
 *
 * Sends soap message to http soap bc and reads same soap message from JMS queue.
 * 
 */
public class HttpBCAsyncTester {
    private ExecutorService threadPool;
    private String url;
    private String queueName;
    private String soapMsg;
    
    class SoapPoster implements Runnable {
        String url;
        String soapMsg;
        public SoapPoster(String url, String soapMsg) {
            this.url = url;
            this.soapMsg = soapMsg;
        }
        public void run() {
            try {
                HttpClientAPI httpC = new HttpClientAPI(url);
                httpC.addContentType("text/xml");
                httpC.setTextData(soapMsg);
                HttpResult result = httpC.post();
                /*
                if (result.getIsTextResult()) {
                    System.out.println ("Got reply ["+result.getTextResult()+"]");
                } else {
                    System.out.println ("Got non-text reply");
                }*/
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        
    }
    
    public HttpBCAsyncTester(String url, String queueName, String soapMsg) {
        this.url = url;
        this.queueName = queueName;
        this.soapMsg = soapMsg;
        this.threadPool = Executors.newCachedThreadPool();
    }

    public void start() {
        Connection conn = null;
        Session sess = null;   
        MessageConsumer consumer = null;
        try {
             
            // Create JMS message consumer
            String port = "7676";
            String host = "localhost";
            ConnectionFactory myConnFactory = new ConnectionFactory();
            myConnFactory.setProperty("imqBrokerHostPort", port);   
            myConnFactory.setProperty("imqBrokerHostName", host);
            conn = myConnFactory.createConnection();
            sess = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
            Destination dest = sess.createQueue(queueName);            
            consumer = sess.createConsumer(dest);
            
            // number of threads on server side to process requests
            //
            // If server side is asynchronous then we expect that all requests are posted and will appear in the JMS queue
            // If server side is synchronous then we expect that some requests are blocked and won't appear in the JMS queue
            int numReqServiceThreads = 3;  
            int numRequestsToSend = numReqServiceThreads + 3;
           
            for (int i=0; i < numRequestsToSend; i++) {
                threadPool.submit(new SoapPoster(url,soapMsg));
            }
            
            // sleep; wait sufficient time for messages to be posted to the queue
            long waitTime = 2 * 1000;
            Thread.currentThread().sleep(waitTime);
            
            // check number of messages in the queue
            int numMsgsInQueue = 0;
            conn.start();
            for (int i=0; i < numRequestsToSend; i++) {
                try {
                    Message msg = consumer.receive(1000);
                    if (msg != null) {
                        numMsgsInQueue++;
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            
            System.out.println ("Number of messages sent - " + numRequestsToSend);
            System.out.println ("Number of messages read from Queue (" + queueName + ") - " + numMsgsInQueue);
            
            List <Runnable> outStandingTasks = threadPool.shutdownNow();
            
        } catch (Exception ex) {
            ex.printStackTrace();;
        } finally {
            if (consumer != null) {
                try {
                    consumer.close();
                } catch (Exception ex) {
                    ex.printStackTrace();;
                }
            }
            if (sess != null) {
                try {
                    sess.close();;
                } catch (Exception ex) {
                    ex.printStackTrace();;
                }            }
            if (conn != null) {
                try {
                    conn.stop();
                    conn.close();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }
        
    }   
    
    public static void main (String [] args) {
        // java HttpBCAsyncTester <url>  <queue>   <f://file|payload>
        String url = args[0];
        String queueName = args[1];
        String soapMsg = args[2];
//        String url = "http://localhost:18181/echoTestService/echoTestPort/Sync";
//        String queueName = "AsyncTestQ";
//        String soapMsg = "f://C:\\TEMP\\HttpBC\\Async\\input\\inputdata.xml";
        
        // Get the send payload
        if (soapMsg.indexOf("f://") >= 0) {
            try {
                String filename = soapMsg.substring(4, soapMsg.length());
                // Read from file
                FileInputStream fis = new FileInputStream(filename);
                StringBuffer strBuff = new StringBuffer();
                byte [] buff = new byte [1024];
                int read = 0;
                read = fis.read(buff);
                while (read != -1) {
                    strBuff.append(new String(buff,0,read));
                    read = fis.read(buff);
                } 
                fis.close();
                
                soapMsg = strBuff.toString();                
	    } catch (Throwable t) {
	        t.printStackTrace();
	        System.exit(1);
	    }            
        }
        
        new HttpBCAsyncTester(url, queueName, soapMsg).start();
    }
}
