/**************************************************************************
 *
 *          Copyright (c) 2002, SeeBeyond Technology Corporation,
 *          All Rights Reserved
 *
 *          This program, and all the routines referenced herein,
 *          are the proprietary properties and trade secrets of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 *          Except as provided for by license agreement, this
 *          program shall not be duplicated, used, or disclosed
 *          without  written consent signed by an officer of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 ***************************************************************************/
package com.stc.connector.httpadapter.http;

import com.sonalb.net.http.cookie.Client;
import com.sonalb.net.http.cookie.Cookie;
import com.sonalb.net.http.cookie.CookieJar;
import com.stc.connector.httpadapter.util.LogUtil;
import org.apache.log4j.Logger;
import java.util.ArrayList;
import java.net.URLConnection;


/**
 * Cookie Manager singleton for managing HTTP cookies.
 *
 * @author $Author: pchan $
 * @version $Version$
 */
public class CookieManager {
    private static CookieManager instance = null;
    private Client jcookieClient = null;
    private CookieJar jcookieJar = null;
    private Logger logger = null;

    /**
     * Creates a new instance of CookieManager
     */
    private CookieManager() {
        jcookieClient = new Client();
        jcookieJar = new CookieJar();
        logger = Logger.getLogger(LogUtil.HTTPCLIENT_CATEGORY + 
            CookieManager.class.getName());
    }

    /**
     * Gets the CookieManager singleton.
     *
     * @return CookieManager    An instance of CookieManager.
     */
    public static CookieManager getInstance() {
        if (instance == null) {
            instance = new CookieManager();
        }

        return instance;
    }

    /**
     * Get the cookies (if any) from the URLConnection and save them.
     *
     * @param urlConn The connected URLConnection instance which may contain
     *        cookies from the server.
     *
     * @throws Exception upon error during cookie processing.
     */
    synchronized public void getCookies(URLConnection urlConn)
        throws Exception {
            
        CookieJar tempJar = jcookieClient.getCookies(urlConn);

        if (tempJar != null && (!tempJar.isEmpty()) ) {
            if (this.jcookieJar == null) {
                this.jcookieJar = new CookieJar(tempJar);
            } else {
                this.jcookieJar.addAll (tempJar);
            }
            
            if (logger.isDebugEnabled()) {
                logger.debug("Getting cookies from URL, " 
                    + urlConn.getURL().toString());

                logger.debug("************** getCookies START COOKIE JAR DUMP *************");
                logger.debug("COOKIE JAR SIZE : " + tempJar.size());

                int cookienum = 1;
                java.util.Iterator cIter = tempJar.iterator();

                while (cIter.hasNext()) {
                    Cookie cookie = (Cookie) cIter.next();
                    logger.debug(
                        "COOKIE #" + cookienum
                        + " [name {" + (cookie.getName()==null?"null":cookie.getName())
                        + "}, value {" + (cookie.getValue()==null?"null":cookie.getValue())
                        + "}, expires {"
                        + (cookie.getExpires()==null?"no expiration or can't parse":cookie.getExpires().toString())
                        + "}, raw {" + cookie.toString() + "}]");
                    cookienum++;
                }
                logger.debug("************** getCookies END COOKIE JAR DUMP *************");
            }
        }
    }

    /**
     * Sets the cookies (if any were previously received) on the connected
     * URLConnection instance.  The cookies will be sent to the server along
     * with the next request.
     *
     * @param urlConn The connected URLConnection instance on which the cookies
     *        will be sent.
     */
    synchronized public void setCookies(URLConnection urlConn) {
        
        if (!jcookieJar.isEmpty()) {
            /**RC get a chance to clean cookie jar
            if (logger.isDebugEnabled()) {
                logger.debug("Setting cookies for URL, " + 
                              urlConn.getURL().toString());
                logger.debug("************** START COOKIE JAR DUMP *************");
                logger.debug("COOKIE JAR SIZE : " + jcookieJar.size());
            }
            **/
             if (logger.isDebugEnabled()) {
             logger.debug("************** setCookies START COOKIE JAR DUMP *************");
             }
            int cookienum = 1;
            java.util.Iterator cIter = jcookieJar.iterator();
            ArrayList cookiesToRemove = new ArrayList();
            while (cIter.hasNext()) {
                Cookie cookie = (Cookie) cIter.next();                
                if (logger.isDebugEnabled()) {
                    logger.debug(
                        "COOKIE #" + cookienum
                        + " [name {" + (cookie.getName()==null?"null":cookie.getName())
                        + "}, value {" + (cookie.getValue()==null?"null":cookie.getValue())
                        + "}, expires {"
                        + (cookie.getExpires()==null?"no expiration or can't parse":cookie.getExpires().toString())
                        + "}, raw {" + cookie.toString() + "}]");
                }
                
                // Remove any expired cookies from the cookie jar so that
                // the cookie would not be sent on the next request.
                if (cookie.hasExpired()) {
                    cookiesToRemove.add (cookie);
                    if (logger.isDebugEnabled()) {
                        logger.debug ("COOKIE #" + cookienum + " [raw {" +
                            cookie.toString() + "}] has EXPIRED; will be removed from cookie jar.");
                    }
                }                
                cookienum++;
            }

            // Remove expired cookies from cookie jar.
            if (cookiesToRemove.size() > 0) {
                for (int i=0; i < cookiesToRemove.size(); i++) {
                    Cookie cookie = (Cookie) cookiesToRemove.get(i);
                    jcookieJar.remove (cookie);                    
                }
                cookiesToRemove.clear();
                cookiesToRemove = null;
            }
                        
            if (logger.isDebugEnabled()) {
                logger.debug("**************setCookies END COOKIE JAR DUMP *************");
            }
            
            /**RC
            jcookieClient.setCookies(
                urlConn,
                jcookieJar);
            **/
            
            /**RC use this cookie jar forever
            if (jcookieJar.isEmpty()) {
                jcookieJar = null;
            }                
             **/
            if (logger.isDebugEnabled()) {
                logger.debug("Setting cookies for URL, " + 
                              urlConn.getURL().toString());            
                logger.debug("COOKIE JAR SIZE : " + jcookieJar.size());
            }
          jcookieClient.setCookies(
                urlConn,
                jcookieJar);
        }
         
    }
}
