/**************************************************************************
 *
 *          Copyright (c) 2002, SeeBeyond Technology Corporation,
 *          All Rights Reserved
 *
 *          This program, and all the routines referenced herein,
 *          are the proprietary properties and trade secrets of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 *          Except as provided for by license agreement, this
 *          program shall not be duplicated, used, or disclosed
 *          without  written consent signed by an officer of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 ***************************************************************************/
package com.stc.connector.httpadapter.http;

import java.io.UnsupportedEncodingException;
import java.security.Principal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import org.apache.log4j.Logger;
import com.stc.connector.httpadapter.util.LogUtil;

// QAI 90061 -- Digest Authentication
/**
 * This class implements the calculations of H(A1), H(A2), 
 * request-digest and response-digest, and a test program which 
 * computes the values used in the example of section 3.5 of the
 * HTTP Authentication: Basic and Digest Access Authentication 
 * (RFC 2617: http://community.roxen.com/developers/idocs/rfc/rfc2617.html). 
 * It uses the MD5 implementation from RFC 1321; the code adapts from the
 * C++ sample in the said document above.
 *
 * @author  $Author: pchan $
 * @version $Revision: 1.1.4.2 $
 */
public class HTTPDigest {

   private String algorithm;   
   
   protected Logger logger = 
      Logger.getLogger(LogUtil.HTTPCLIENT_CATEGORY + getClass().getName()); // TODO: client or server or both
      
   public static final String DIGESTENCODINGSUPPORTED = "ISO-8859-1";
   public static final String MD5_SESS = "md5-sess";
   
   public static String CvtHex(byte Bin[])
   {
       int i;
       StringBuffer hexStringBuf = new StringBuffer();
       String hexString = "";
   
       for (i = 0; i < Bin.length; i++) {
           hexString = Integer.toHexString(0xFF & Bin[i]);
           if (hexString.length() == 1) {
               // Pad with zero manually because toHexString doesn't, sigh!
               hexString = "0" + hexString;
           }        
           hexStringBuf.append(hexString);
       }
   
       return (hexStringBuf.toString());
   }
   
   // calculate H(A1) as per spec
   public String DigestCalcHA1(String pszAlg,
                               String pszUserName,
                               String pszRealm,
                               String pszPassword,
                               String pszNonce,
                               String pszCNonce
                              ) throws Exception {

         if ( (pszUserName == null) ||
              (pszRealm == null) ||
              (pszPassword == null) ) {
            throw new Exception("DigestCalcHA1: User name, password, or realm is null or empty!");
         }

         StringBuffer Md5Ctx = new StringBuffer(256);
         String HA1;
        
         Md5Ctx.append(pszUserName);
         Md5Ctx.append(":");
         Md5Ctx.append(pszRealm);
         Md5Ctx.append(":");
         Md5Ctx.append(pszPassword);
         HA1 = Md5Ctx.toString();
   
         // MD5 session
         if (pszAlg.equalsIgnoreCase(MD5_SESS)) {
               Md5Ctx.append(HA1);
               Md5Ctx.append(":");
               Md5Ctx.append(pszNonce);
               Md5Ctx.append(":");
               Md5Ctx.append(pszCNonce);
         }

         MessageDigest md = null;
         try {
            md = getMessageDigest(pszAlg);
            if (md != null) {
               md.update(HA1.getBytes(DIGESTENCODINGSUPPORTED));
               HA1 = CvtHex(md.digest());
               md.reset();
            }
         } catch (UnsupportedEncodingException usenex) {
            // in case iso-8859-1 is not known...
            this.logger.error("DigestCalcHA1: exception caught: " + usenex.getMessage());
            throw new Exception(usenex.getMessage());

         } catch (Exception ex) {
            throw new Exception(ex.getMessage());
         }

         return HA1;
   }
   
   // calculate request-digest/response-digest as per HTTP Digest spec //
   public String DigestCalcResponse(
       String HA1,
       String pszNonce,       // nonce from server //
       String pszNonceCount,  // 8 hex digits //
       String pszCNonce,      // client nonce //
       String pszQop,         // qop-value: "", "auth", "auth-int" //
       String pszMethod,      // method from the request //
       String pszDigestUri,   // requested URL //
       String HEntity,
       String pszAlg
       ) throws Exception {

         StringBuffer Md5Ctx = new StringBuffer(256);
         String HA2;
         String RespHash;
   
         // calculate H(A2)
   
         Md5Ctx.append(pszMethod);
         
         Md5Ctx.append(":");
         Md5Ctx.append(pszDigestUri);
         if (pszQop.equalsIgnoreCase("auth-int") == true) {
               Md5Ctx.append(":");
               Md5Ctx.append(HEntity);
         }
         HA2 = Md5Ctx.toString();

         if (this.logger.isDebugEnabled() == true) {
             this.logger.debug("1: DigestCalcResponse: HA2=" + HA2);
         }
         
         MessageDigest md = null;
         try {
            md = getMessageDigest(pszAlg); 
            if (md != null) {
    	         md.update(HA2.getBytes(DIGESTENCODINGSUPPORTED));
    	      }
         } catch (UnsupportedEncodingException usenex) {
            // in case iso-8859-1 is not known...
            this.logger.error("DigestCalcResponse: exception caught: " + usenex.getMessage());
            throw new Exception(usenex.getMessage());

         } catch (Exception ex) {
            throw new Exception(ex.getMessage());
         }
   
   	   HA2 = CvtHex(md.digest());

         if (this.logger.isDebugEnabled() == true) {
             this.logger.debug("2: DigestCalcResponse: HA2=" + HA2);
         }

   	   md.reset();
   	         
         // calculate response
         Md5Ctx = new StringBuffer(256);
         Md5Ctx.append(HA1);
         Md5Ctx.append(":");
         Md5Ctx.append(pszNonce);
         Md5Ctx.append(":");
         if (pszQop != null && pszQop.length() >0) {
             Md5Ctx.append(pszNonceCount);
             Md5Ctx.append(":");
             Md5Ctx.append(pszCNonce);
             Md5Ctx.append(":");
             Md5Ctx.append(pszQop);
             Md5Ctx.append(":");
         }
         Md5Ctx.append(HA2);
         
         HA2 = Md5Ctx.toString();
         
         if (this.logger.isDebugEnabled() == true) {
             this.logger.debug("3: DigestCalcResponse: HA2=" + HA2);
         }
   
         try {
      	   md.update(HA2.getBytes("ISO-8859-1"));
         } catch (UnsupportedEncodingException usenex) {
            // in case iso-8859-1 is not known...
            this.logger.error("DigestCalcResponse: exception caught: " + usenex.getMessage());
            throw new Exception(usenex.getMessage());
         }
   
   	   RespHash = CvtHex(md.digest());
   	   
   	   return RespHash;
   	   
   }

   private MessageDigest getMessageDigest(String pszAlg) throws Exception {

      MessageDigest md = null;

      try {
         String algorithm_to_use = "";
         if (pszAlg == null || pszAlg.length() <= 0) {
            if (this.algorithm != null && this.algorithm.length() > 0) {
               if (!this.algorithm.equalsIgnoreCase(MD5_SESS)) {
                  algorithm_to_use = this.algorithm;
               } else {
                  algorithm_to_use = "md5"; // md5-sess -> md5
               }
            } else {
               algorithm_to_use = "md5"; // the default
            }
         } else {
            algorithm_to_use = pszAlg;
         }
      
         md = MessageDigest.getInstance(algorithm_to_use);
      
      } catch (NoSuchAlgorithmException algex) {

         this.logger.error("getMessageDigest: exception caught: NoSuchAlgorithmException: " + algex.getMessage());
         throw new Exception(algex.getMessage());
      }
      
      return md;
   }         
         
   public HTTPDigest(String algorithm) {

      this.algorithm = algorithm;
      // TODO: other constructor parameters needed?
   }

   public static void main(String args[]) {
      
      // Test1
      String pszNonce = "dcd98b7102dd2f0e8b11d0f600bfb0c093";
      String pszCNonce = "0a4f113b";
      String pszUser = "Mufasa";
      String pszRealm = "testrealm@host.com";
      String pszPass = "Circle Of Life";
      String pszAlg = "md5";
      String szNonceCount = "00000001";
      String pszMethod = "GET";
      String pszQop = "auth"; /* "auth-int"; */ /*  "auth,auth-int"; */
      String pszURI = "/dir/index.html";
      String HA1;
      String HA2 = "";
      String Response;
      
      HTTPDigest httpDigest = new HTTPDigest("md5");
      
      try {
         HA1 = httpDigest.DigestCalcHA1(pszAlg, pszUser, pszRealm, pszPass, pszNonce, pszCNonce);
         if (HA1 != null) {
            System.out.println("*******************************************");
            System.out.println("HA1 for Digest for test case 1 = \n" + HA1);
            
            Response = httpDigest.DigestCalcResponse(HA1, pszNonce, szNonceCount, pszCNonce, pszQop, pszMethod, pszURI, HA2, pszAlg);

            // Response should be:
            //
            //    6629fae49393a05397450978507c4ef1
            //
            // according to http://community.roxen.com/developers/idocs/rfc/rfc2617.html
            //
            System.out.println("Response for Digest for test case 1 = \n" + Response);

         } else {
            System.out.println("Error in computing HA1 for test case 1");
         }

               
      } catch (Exception e) {
         System.out.println("Exception caught for DigestCalcHA1 or DigestCalcResponse for test case 1: " + e.getMessage());
      }
      
      // Test2: no nonce, is it practical?
      pszNonce = "";
      pszCNonce = "";
      pszUser = "Mufasa";
      pszRealm = "testrealm@host.com";
      pszPass = "Circle Of Life";
      pszAlg = "";
      szNonceCount = "";
      pszMethod = "GET";
      pszQop = "";
      pszURI = "";
      HA2 = "";

      httpDigest = new HTTPDigest("md5");
      
      try {
         HA1 = httpDigest.DigestCalcHA1(pszAlg, pszUser, pszRealm, pszPass, pszNonce, pszCNonce);
         if (HA1 != null) {
            System.out.println("*******************************************");
            System.out.println("HA1 for Digest for test case 2 = \n" + HA1);

            Response = httpDigest.DigestCalcResponse(HA1, pszNonce, szNonceCount, pszCNonce, pszQop, pszMethod, pszURI, HA2, pszAlg);

            System.out.println("Response for Digest for test case 2 = \n" + Response);
            
         } else {
            System.out.println("Error in computing HA1 for test case 2");
         }
      
      } catch (Exception e) {
         System.out.println("Exception caught for DigestCalcHA1 or DigestCalcResponse for test case 2: " + e.getMessage());
      }

   }

}