/**************************************************************************
 *
 *          Copyright (c) 2002, SeeBeyond Technology Corporation,
 *          All Rights Reserved
 *
 *          This program, and all the routines referenced herein,
 *          are the proprietary properties and trade secrets of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 *          Except as provided for by license agreement, this
 *          program shall not be duplicated, used, or disclosed
 *          without  written consent signed by an officer of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 ***************************************************************************/
package com.stc.connector.httpadapter.http;

import java.net.Authenticator;
import java.net.InetAddress;
import java.net.PasswordAuthentication;
import java.net.UnknownHostException;


/**
 * This class handles the standard HTTP Authentication request
 * from web sites that require username and password authentication.
 *
 * @author  $Author: pchan $
 * @version $Revision: 1.2.134.2 $
 */
public class HttpAuthenticator
    extends Authenticator {
    private String proxyHost = null;
    private String proxyUserName = null;
    private String proxyPassWord = null;
    private String httpUserName = null;
    private String httpPassWord = null;
    private InetAddress proxyInetAddr = null;

    /**
     * Constructs an HttpAuthenticator
     */
    public HttpAuthenticator() {
    }

    /**
     * Constructs an HttpAuthenticator
     *
     * @param userName DOCUMENT ME!
     * @param passWord DOCUMENT ME!
     */
    public HttpAuthenticator(String userName, String passWord) {
        httpUserName = userName;
        httpPassWord = passWord;
    }

    /**
     * Sets the Proxy host so that when proxy authentication is requested then
     * this Authenticator can send the appropriate username and password to
     * this Proxy Host.
     *
     * @param proxyHost The host name or IP address (dotted quad address)
     *
     * @throws UnknownHostException Thrown when IP address or host provided can
     *         not be determined.
     */
    public void setProxyHost(String proxyHost)
        throws UnknownHostException {
        this.proxyHost = proxyHost;
        this.proxyInetAddr = InetAddress.getByName(this.proxyHost);
    }

    /**
     * Sets the Proxy username for Proxy authentication.
     *
     * @param proxyUserName The username for Proxy authentication.
     */
    public void setProxyUserName(String proxyUserName) {
        this.proxyUserName = proxyUserName;
    }

    /**
     * Sets the Proxy password for Proxy authentication.
     *
     * @param proxyPassWord The username for Proxy authentication.
     */
    public void setProxyPassWord(String proxyPassWord) {
        this.proxyPassWord = proxyPassWord;
    }

    /**
     * Sets the username for HTTP authentication.
     *
     * @param httpUserName The username for HTTP authentication.
     */
    public void setHttpUserName(String httpUserName) {
        this.httpUserName = httpUserName;
    }

    /**
     * Sets the password for HTTP authentication.
     *
     * @param httpPassWord The password for HTTP authentication.
     */
    public void setHttpPassWord(String httpPassWord) {
        this.httpPassWord = httpPassWord;
    }

    /**
     * Registers this HTTP authentication instance for Proxy and/or HTTP
     * authentication.
     *
     * @throws SecurityException upon error.
     */
    public void register()
        throws SecurityException {

        // Now register this instance for HTTP and/or Proxy authentication.
        Authenticator.setDefault(this);
    }

    /**
     * UnRegisters/Disable this HTTP authentication instance for Proxy and/or HTTP
     * authentication by setting to the default to null per Authenticator Javadoc.
     *
     * @throws SecurityException upon error.
     */
    public void unregister()
        throws SecurityException {

        // Now unregister this instance for HTTP and/or Proxy authentication; 
        // any previously set authenticator is removed if the default is null.
        Authenticator.setDefault(null);
    }

    /**
     * Returns the PasswordAuthentication object.  This method overrides the
     * Authenticator getPasswordAuthentication method.  If the requesting site
     * is the proxy then this method will return the PasswordAuthentication
     * object such that it contains the username and password set with the
     * method setProxyUserName and setProxyPassWord.  Otherwise, this method
     * will return the PasswordAuthentication object
     *
     * @return DOCUMENT ME!
     */
    protected PasswordAuthentication getPasswordAuthentication() {
        InetAddress requesterAddress = getRequestingSite();

        // If requesting site is proxy, then return proxy P
        // asswordAuthentication object.
        if ((proxyInetAddr != null) && (
                    proxyInetAddr.equals(requesterAddress)
                )) {
            if ((proxyUserName == null) && (proxyPassWord != null)) {
                return new PasswordAuthentication(
                    "NoProxyUserSpecified",
                    proxyPassWord.toCharArray());
            } else if ((proxyUserName != null) && (proxyPassWord == null)) {
                return new PasswordAuthentication(
                    proxyUserName,
                    "NoProxyPassSpecified".toCharArray());
            } else if ((proxyUserName == null) && (proxyPassWord == null)) {
                return new PasswordAuthentication(
                    "NoProxyUserSpecified",
                    "NoProxyPassSpecified".toCharArray());
            }

            //return proxyAuthentication;
            return new PasswordAuthentication(
                proxyUserName,
                proxyPassWord.toCharArray()); // return new instance every time.
        } else {
            if ((httpUserName == null) && (httpPassWord != null)) {
                return new PasswordAuthentication(
                    "NoHttpUserSpecified",
                    httpPassWord.toCharArray());
            } else if ((httpUserName != null) && (httpPassWord == null)) {
                return new PasswordAuthentication(
                    httpUserName,
                    "NoHttpPassSpecified".toCharArray());
            } else if ((httpUserName == null) && (httpPassWord == null)) {
                return new PasswordAuthentication(
                    "NoHttpUserSpecified",
                    "NoHttpPassSpecified".toCharArray());
            }

            //return httpAuthentication;
            return new PasswordAuthentication(
                httpUserName,
                httpPassWord.toCharArray());
        }
    }
}
