/**************************************************************************
 *
 *          Copyright (c) 2002, SeeBeyond Technology Corporation,
 *          All Rights Reserved
 *
 *          This program, and all the routines referenced herein,
 *          are the proprietary properties and trade secrets of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 *          Except as provided for by license agreement, this
 *          program shall not be duplicated, used, or disclosed
 *          without  written consent signed by an officer of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 ***************************************************************************/
/*
 * HttpClientAPIJDKPlatform.java
 *
 * Created on January 13, 2003, 5:32 PM
 */
package com.stc.connector.httpadapter.http;

import java.io.IOException;
import java.net.HttpURLConnection;


/**
 * Wrapper interface for JDK specific SSL/JSSE methods.
 *
 * @author $Author: jtran $
 * @version $Revision: 1.4 $
 */
public interface HttpClientAPIJDKPlatform {
    /**
     * Set the SSLSocketFactory for the current HttpURLConnection using the
     * SSLContext provided if the connection is an HTTPS connection.
     *
     * @param urlConn The HttpURLConnection for the HTTPS connection.
     * @param sslContext The SSLContext which provides key management
     *        SSLSocketFactory.
     *
     * @return The boolean true if the connection is an HTTPS connection and
     *         the SSLSocketFactory was set for the connection using the
     *         SSLSocketFactory provided by the SSLContext.
     *
     * @throws IOException upon error.
     */
    public boolean setSocketFactory(HttpURLConnection urlConn,
        SSLContextJDKPlatform sslContext)
        throws IOException;

    /**
     * Set the SSLTunnelSocketFactory for the current HttpURLConnection using
     * the SSLContext provided if the connection is an HTTPS connection.
     *
     * @param urlConn The HttpURLConnection for the HTTPS connection.
     * @param sslContext The SSLContext which provides key management
     *        SSLSocketFactory.
     * @param host The proxy host name.
     * @param port The proxy port number.
     * @param proxyUserName The proxy username.
     * @param proxyPassWord The proxy password.
     *
     * @return The boolean true if the connection is an HTTPS connection and
     *         the SSLTunnelSocketFactory was set for the connection using the
     *         SSLSocketFactory provided by the SSLContext.
     *
     * @throws IOException upon error.
     */
    public boolean setTunnelSocketFactory(HttpURLConnection urlConn,
        SSLContextJDKPlatform sslContext, String host, int port,
        String proxyUserName, String proxyPassWord)
        throws IOException;

    /**
     * Sets the connection to not do hostname verification on the
     * X509 certificate if the connection is an https connection.
     *
     * @param urlConn The HttpURLConnection for the HTTP connection.
     *
     * @return True if connection is https connection and no hostname
     *         verifier was set successfully; otherwise false.
     * 
     * @throws Exception upon error.
     */
    public boolean setNoHostnameVerifier(HttpURLConnection urlConn)
        throws Exception;
}
