/**************************************************************************
 *
 *          Copyright (c) 2002, SeeBeyond Technology Corporation,
 *          All Rights Reserved
 *
 *          This program, and all the routines referenced herein,
 *          are the proprietary properties and trade secrets of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 *          Except as provided for by license agreement, this
 *          program shall not be duplicated, used, or disclosed
 *          without  written consent signed by an officer of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 ***************************************************************************/
package com.stc.connector.httpadapter.http;

import java.util.Vector;


/**
 * DOCUMENT ME!
 *
 * @author James D. Tran
 * @version $Revision: 1.4 $
 */
public class HttpResult {
    int responseCode = -1;
    String responseMessage = null;
    String textResult = null;
    Vector headers = null;
    byte[] byteResult = null;
    boolean isTextData = true;

    /**
     * Constructs an HttpResult
     */
    public HttpResult() {
        headers = new Vector();
    }

    /**
     * Sets the response code returned from the server
     *
     * @param responseCode The HTTP response code from the server.
     */
    public void setResponseCode(int responseCode) {
        this.responseCode = responseCode;
    }

    /**
     * Gets the response code returned from the server
     *
     * @return The HTTP response code from the server.
     *
     * @throws Exception DOCUMENT ME!
     */
    public int getResponseCode()
        throws Exception {
        if (responseCode == -1) {
            throw new Exception("No response code to retrieve.");
        } else {
            return responseCode;
        }
    }

    /**
     * Sets the response message returned from the server
     *
     * @param responseMessage The HTTP response Message from the server.
     */
    public void setResponseMessage(String responseMessage) {
        this.responseMessage = responseMessage;
    }

    /**
     * Gets the response message returned from the server
     *
     * @return The HTTP response Message from the server.
     *
     * @throws Exception DOCUMENT ME!
     */
    public String getResponseMessage()
        throws Exception {
            return responseMessage;
    }

    /**
     * Sets the response message returned from the server as text if given
     * true; otherwise set the response message returned from server as binary
     * if given false.
     *
     * @param isTextData The flag to indicate text data if set to true;
     *        otherwise binary data if set to false.
     */
    public void setIsTextResult(boolean isTextData) {
        this.isTextData = isTextData;
    }

    /**
     * Gets the data type returned from the server; if the data is text then
     * true is returned; otherwise false is returned for binary data.
     *
     * @return If data received from the server is text data, then true is
     *         returned.  Otherwise, false is returned to indicate binary
     *         data.
     *
     * @throws Exception DOCUMENT ME!
     */
    public boolean getIsTextResult()
        throws Exception {
        if ((textResult == null) && (byteResult == null)) {
            throw new Exception(
                "Can not determine result type because there is no result");
        } else {
            return isTextData;
        }
    }

    /**
     * Sets the binary result returned from the server
     *
     * @param textResult The HTTP text result from the server.
     */
    public void setTextResult(String textResult) {
        this.textResult = textResult;
    }

    /**
     * Gets the text result returned from the server
     *
     * @return The HTTP text result from the server.
     *
     * @throws Exception DOCUMENT ME!
     */
    public String getTextResult()
        throws Exception {
        if (textResult == null) {
            throw new Exception("No text result to retrieve.");
        } else {
            return textResult;
        }
    }

    /**
     * Sets the binary result returned from the server
     *
     * @param binaryResult The HTTP binary result from the server.
     */
    public void setBinaryResult(byte[] binaryResult) {
        byteResult = binaryResult;
    }

    /**
     * Gets the binary result returned from the server
     *
     * @return The HTTP binary result from the server.
     *
     * @throws Exception DOCUMENT ME!
     */
    public byte[] getBinaryResult()
        throws Exception {
        if (byteResult == null) {
            throw new Exception("No binary result to retrive.");
        } else {
            return byteResult;
        }
    }

    /**
     * Sets the result headers returned from the server
     *
     * @param headers HTTP result headers from the server in a Vector.
     */
    public void setHeaders(Vector headers) {
        this.headers = new Vector(headers);
    }

    /**
     * Adds a result header to the list of headers.
     *
     * @param header The HTTP result header from the server.
     */
    public void addHeader(HttpHeader header) {
        headers.add(new HttpHeader(
                header.getName(),
                header.getValue()));
    }

    /**
     * Gets a count of the headers.
     *
     * @return The number of result headers from the server.
     */
    public int getHeaderCount() {
        if (headers == null) {
            return 0;
        } else {
            return headers.size();
        }
    }

    /**
     * Gets a header from the list of headers.
     *
     * @param index The index of the header to retrieve.
     *
     * @return A result header from the server.
     *
     * @throws Exception DOCUMENT ME!
     */
    public HttpHeader getHeader(int index)
        throws Exception {
        if (index >= headers.size()) {
            throw new Exception("Index " + index
                + " is out of range. Number of headers is " + getHeaderCount()
                + ".");
        } else {
            return new HttpHeader(
                ((HttpHeader) headers.get(index)).getName(),
                ((HttpHeader) headers.get(index)).getValue());
        }
    }
}
