/**************************************************************************
 *
 *          Copyright (c) 2002, SeeBeyond Technology Corporation,
 *          All Rights Reserved
 *
 *          This program, and all the routines referenced herein,
 *          are the proprietary properties and trade secrets of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 *          Except as provided for by license agreement, this
 *          program shall not be duplicated, used, or disclosed
 *          without  written consent signed by an officer of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 ***************************************************************************/
package com.stc.connector.httpadapter.http;

/**
 * DOCUMENT ME!
 *
 * @author James D. Tran
 * @version $Revision: 1.2 $
 */
public final class HttpsSystemProperties {
    /** HTTPS protocol handler property */
    protected static final String HTTPS_PROTOCOL_IMPL_PROPERTY = "java.protocol.handler.pkgs";

    // Not JSSE standard.. may change in the future.
    /** Default keystore property */
    protected static final String DEFAULT_KEYSTORE = "javax.net.ssl.keyStore";
    /** Default keystore type property */
    protected static final String DEFAULT_KEYSTORE_TYPE = "javax.net.ssl.keyStoreType";
    /** Default keystore password property */
    protected static final String DEFAULT_KEYSTORE_PASSWORD = "javax.net.ssl.keyStorePassword";
    /** Default truststore property */
    protected static final String DEFAULT_TRUSTSTORE = "javax.net.ssl.trustStore";
    /** Default truststore type property */
    protected static final String DEFAULT_TRUSTSTORE_TYPE = "javax.net.ssl.trustStoreType";
    /** Default truststore password property */
    protected static final String DEFAULT_TRUSTSTORE_PASSWORD = "javax.net.ssl.trustStorePassword";

    /**
     * Sets the HTTPs protocol implementation.   This method will add the
     * "https"  URLStreamHandler implementation by including the handler's
     * implementation  package name to the list of packages which are searched
     * by the Java URL class.
     *
     * @param httpsProtocolImpl The HTTPs protocol implementation package name.
     *        For example, to use SUN's HTTPs protocol handler reference
     *        implementation, specify "com.sun.net.ssl.internal.www.protocol".
     *
     * @throws Exception is thrown if unable to set the HTTPs Protocol
     *         Implementation.
     */
    public static void setHttpsProtocolImpl(String httpsProtocolImpl)
        throws Exception {
        try {
            System.setProperty(
                HTTPS_PROTOCOL_IMPL_PROPERTY,
                httpsProtocolImpl);
        } catch (Exception ex) {
            throw new Exception(
                "Failed to set HTTPs Protocol Implementation; Exception : "
                + ex.toString());
        }
    }

    /**
     * Gets the HTTPs protocol implementation that was previously set.
     *
     * @return The HTTPs protocol implementation package name. For example, if
     *         using the SUN's HTTPs protocol handler reference
     *         implementation, it will return
     *         "com.sun.net.ssl.internal.www.protocol". If not previously set,
     *         null will be returned.
     */
    public static String getHttpsProtocolImpl() {
        return System.getProperty(HTTPS_PROTOCOL_IMPL_PROPERTY);
    }

    //////////////////////////////
    // Everyting below here may not guaranteed to continue to have the same names 
    // and types (system or security) or even to exist at all in future releases. 
    // They are also not guaranteed to be examined and used by any other JSSE 1.0.2 
    // implementations. If they are examined by an implementation, that implementation 
    // should handle them in the same manner as the JSSE 1.0.2 reference implementation 
    // does, as described in the JSSE documentation.
    //////////////////////////////
    /**
     * Sets the default key store file. If the default key store is not
     * specified with this method, then  the key store managed by KeyManager
     * is empty.
     *
     * @param keyStoreFile Full path file name specifying the key store. For
     *        example, "C:\mykeystores\keystore1". See the keytool utitlity
     *        program from SUN for more detail.
     *
     * @throws Exception if unable to set the default key store.
     */
    public static void setKeyStore(String keyStoreFile)
        throws Exception {
        try {
            System.setProperty(
                DEFAULT_KEYSTORE,
                keyStoreFile);
        } catch (Exception ex) {
            throw new Exception(
                "Failed to set the default key store; Exception : "
                + ex.toString());
        }
    }

    /**
     * Gets the default key store previously set.
     *
     * @return Full path file name specifying the key store if previously set;
     *         otherwise, returns null.
     */
    public static String getKeyStore() {
        return System.getProperty(DEFAULT_KEYSTORE);
    }

    /**
     * Sets the default key store type. If the default key store type is not
     * set with this method, then the default key store type "jks" is used.
     *
     * @param keyStoreType The key store type. For example, "jks".
     *
     * @throws Exception if unable to set the default key store type.
     */
    public static void setKeyStoreType(String keyStoreType)
        throws Exception {
        try {
            System.setProperty(
                DEFAULT_KEYSTORE_TYPE,
                keyStoreType);
        } catch (Exception ex) {
            throw new Exception(
                "Failed to set the default key store type; Exception : "
                + ex.toString());
        }
    }

    /**
     * Gets the default key store type previously set.
     *
     * @return The key store type that was previously set.  Null if not
     *         previously set.
     */
    public static String getKeyStoreType() {
        return System.getProperty(DEFAULT_KEYSTORE_TYPE);
    }

    /**
     * Sets the default key store password. If the default key store password
     * is not set with this method, then the default key store password is
     * assumed to be "". See the keytool from SUN for more detail
     *
     * @param keyStorePassword The key store password.
     *
     * @throws Exception if unable to set the default key store password.
     */
    public static void setKeyStorePassword(String keyStorePassword)
        throws Exception {
        try {
            System.setProperty(
                DEFAULT_KEYSTORE_PASSWORD,
                keyStorePassword);
        } catch (Exception ex) {
            throw new Exception(
                "Failed to set the default key store password; Exception : "
                + ex.toString());
        }
    }

    /**
     * Gets the default key store password previously set.
     *
     * @return The key store password that was previously set.  Null if not
     *         previously set.
     */
    public static String getKeyStorePassword() {
        return System.getProperty(DEFAULT_KEYSTORE_PASSWORD);
    }

    /**
     * Sets the default trust store. If the default trust store is not
     * specified with this method, then a default  truststore is searched for.
     * If a truststore named java-home/lib/security/jssecacerts  is found, it
     * is used. If not, then a truststore named java-home/lib/security/cacerts
     * is searched for and used (if it exists). Finally, if a truststore is
     * still not found,  then the truststore managed by the TrustManager will
     * be a new empty truststore.  See the keytool utitlity program from SUN
     * for more detail.
     *
     * @param trustStoreFile Full path file name specifying the trust store.
     *
     * @throws Exception if unable to set the default trust store.
     */
    public static void setTrustStore(String trustStoreFile)
        throws Exception {
        try {
            System.setProperty(
                DEFAULT_TRUSTSTORE,
                trustStoreFile);
        } catch (Exception ex) {
            throw new Exception(
                "Failed to set the default trust store; Exception : "
                + ex.toString());
        }
    }

    /**
     * Gets the default trust store previously set.
     *
     * @return Full path file name specifying the trust store if previously
     *         set.
     */
    public static String getTrustStore() {
        return System.getProperty(DEFAULT_TRUSTSTORE);
    }

    /**
     * Sets the default trust store type.
     *
     * @param trustStoreType The trust store type.
     *
     * @throws Exception if unable to set the default key store type.
     */
    public static void setTrustStoreType(String trustStoreType)
        throws Exception {
        try {
            System.setProperty(
                DEFAULT_TRUSTSTORE_TYPE,
                trustStoreType);
        } catch (Exception ex) {
            throw new Exception(
                "Failed to set the default trust store type; Exception : "
                + ex.toString());
        }
    }

    /**
     * Gets the default trust store type previously set.
     *
     * @return The trust store type that was previously set.  Null if not
     *         previously set.
     */
    public static String getTrustStoreType() {
        return System.getProperty(DEFAULT_TRUSTSTORE_TYPE);
    }

    /**
     * Sets the default trust store password. If the default trust store
     * password is not set with this method, then the default trust store
     * password is assumed to be "". See the keytool from SUN for more detail
     *
     * @param trustStorePassword The trust store password.
     *
     * @throws Exception if unable to set the default trust store password.
     */
    public static void setTrustStorePassword(String trustStorePassword)
        throws Exception {
        try {
            System.setProperty(
                DEFAULT_TRUSTSTORE_PASSWORD,
                trustStorePassword);
        } catch (Exception ex) {
            throw new Exception(
                "Failed to set the default trust store password; Exception : "
                + ex.toString());
        }
    }

    /**
     * Gets the default trust store password previously set.
     *
     * @return The trust store password that was previously set.  Null if not
     *         previously set.
     */
    public static String getTrustStorePassword() {
        return System.getProperty(DEFAULT_TRUSTSTORE_PASSWORD);
    }
}
