/**************************************************************************
 *
 *          Copyright (c) 2002, SeeBeyond Technology Corporation,
 *          All Rights Reserved
 *
 *          This program, and all the routines referenced herein,
 *          are the proprietary properties and trade secrets of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 *          Except as provided for by license agreement, this
 *          program shall not be duplicated, used, or disclosed
 *          without  written consent signed by an officer of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 ***************************************************************************/
package com.stc.connector.httpadapter.http;

import java.util.Vector;


/**
 * DOCUMENT ME!
 *
 * @author James D. Tran
 * @version $Revision: 1.2 $
 */
public class QueryString {
    /** Collection of QueryPairs */
    protected Vector queryPairs;

    /** Placeholder for QueryPair object for use
        with java collab edtor for drag and drop feature */
    protected QueryPair qPair;

    /**
     * Constructs an empty QueryString
     */
    public QueryString() {
        // Create collection object.
        queryPairs = new Vector();

        // Create QueryPair object placeholder. yuck.
        qPair = new QueryPair();
    }

    /**
     * Constructs a QueryString with an exiting QueryPair vector and a
     * temporary QueryPair object placeholder.
     *
     * @param queryPairs DOCUMENT ME!
     * @param qPair DOCUMENT ME!
     */
    public QueryString(Vector queryPairs, QueryPair qPair) {
        this.queryPairs = (Vector) queryPairs.clone();

        String qName = qPair.getName();
        String qValue = qPair.getValue();

        if ((qName == null) || (qValue == null)) {
            this.qPair = new QueryPair();
        } else {
            this.qPair = new QueryPair(
                    qPair.getName(),
                    qPair.getValue());
        }
    }

    /**
     * Constructs a QueryString with an intial QueryPair.
     *
     * @param queryPair The query name/value pair
     *
     * @see QueryPair
     */
    public QueryString(QueryPair queryPair) {
        // Create Collection object and add initial QueryPair object.
        queryPairs = new Vector();
        queryPairs.add(queryPair);

        // Create QueryPair object placeholder. yuck.
        qPair = new QueryPair();
    }

    /**
     * Constructs a QueryString with an intial name value pair.
     *
     * @param name The name portion of the query pair.
     * @param value The value portion of the query pair.
     *
     * @see QueryPair
     */
    public QueryString(String name, String value) {
        queryPairs = new Vector();
        queryPairs.add(new QueryPair(
                name,
                value));
    }

    /**
     * Constructs a QueryString by cloning itself.
     *
     * @return The cloned QueryString object. Needs to be casted to a
     *         QueryString as this method overrides the Object clone() method.
     */
    public Object clone() {
        return new QueryString(
            this.queryPairs,
            this.qPair);
    }

    /**
     * Sets the QueryPair placeholder object.  Added to work with xsc and java
     * collab editor for drag and drop. Once data is populated in the
     * QueryPair placeholder object, use one of the other methods for managing
     * query or form data information. If this method is not called, a
     * default QueryPair objectis used.
     *
     * @param qPair The QueryPair object to be used as a place holder for
     *        adding HTTP query or form data information.
     *
     * @see #getQueryPair
     * @see #add
     * @see QueryPair
     */
    public void setQueryPair(QueryPair qPair) {
        this.qPair = new QueryPair(
                qPair.getName(),
                qPair.getValue());
    }

    /**
     * Gets the QueryPair placeholder object.  Added to work with xsc and java
     * collab editor for drag and drop. Once data is populated in the
     * QueryPair placeholder object, use one of the other methods for managing
     * query or form data information. If this method is not called, a
     * default QueryPair objectis used.
     *
     * @return The QueryPair object being used as a place  holder for adding
     *         HTTP query or form data  information.
     *
     * @see #setQueryPair
     * @see #add
     * @see QueryPair
     */
    public QueryPair getQueryPair() {
        return qPair;
    }

    /**
     * Adds a name/value pairing.
     *
     * @param queryPair The query name/value pair
     *
     * @see QueryPair
     */
    public synchronized void add(QueryPair queryPair) {
        queryPairs.add(
            new QueryPair(
                queryPair.getName(),
                queryPair.getValue()));
    }

    /**
     * Adds a name/value pairing.
     *
     * @param name The name portion of the query pair.
     * @param value The value portion of the query pair.
     */
    public synchronized void add(String name, String value) {
        queryPairs.add(new QueryPair(
                name,
                value));
    }

    /**
     * Returns the string format of QueryString
     *
     * @return The string format of QueryString; the name and value pairs of
     *         each query will be URL encoded.
     */
    public String toString() {
        String queryString = "";

        for (int i = 0; i < queryPairs.size(); i++) {
            if (!queryString.trim().equals("")) {
                queryString += "&";
            }

            queryString += ((QueryPair) queryPairs.get(i)).toString();
        }

        return queryString;
    }

    /**
     * Returns the string format of QueryString
     *
     * @return The string format of QueryString; the name and value pairs of
     *         each query will be URL encoded.
     */
    public String getQueryString() {
        String queryString = null;

        for (int i = 0; i < queryPairs.size(); i++) {
            if (queryString != null) {
                queryString += "&";
            }

            queryString += ((QueryPair) queryPairs.get(i)).toString();
        }

        return new String(queryString);
    }

    /**
     * Gets a count of the query pairs
     *
     * @return The number of query pairs in the query
     */
    public int getCount() {
        return queryPairs.size();
    }

    /**
     * Gets a query pair from the list of query pairs
     *
     * @param index The index to the QueryPair that is to be retrieved.
     *
     * @return A query pair in the query
     *
     * @see QueryPair
     */
    public QueryPair getQueryPair(int index) {
        return new QueryPair(
            ((QueryPair) queryPairs.get(index)).getName(),
            ((QueryPair) queryPairs.get(index)).getValue());
    }
}
