/**************************************************************************
 *
 *          Copyright (c) 2002, SeeBeyond Technology Corporation,
 *          All Rights Reserved
 *
 *          This program, and all the routines referenced herein,
 *          are the proprietary properties and trade secrets of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 *          Except as provided for by license agreement, this
 *          program shall not be duplicated, used, or disclosed
 *          without  written consent signed by an officer of
 *          SEEBEYOND TECHNOLOGY CORPORATION.
 *
 ***************************************************************************/
package com.stc.connector.httpadapter.http;

import java.io.IOException;
import javax.net.ssl.SSLSocketFactory;


/**
 * Interface for JDK specific SSLContext wrapper.
 *
 * @author $Author: jtran $
 * @version $Revision: 1.3 $
 */
public interface SSLContextJDKPlatform {
    /**
     * Initialize the underlying SSLContext.
     *
     * @param sslProtocol The SSL Protocol to use. Valid are SSL, SSLv2, SSLv3,
     *        TLS, TLSv1
     * @param trustStoreType The truststore type (i.e., JKS).
     * @param trustStore The truststore location.
     * @param trustStorePassword The truststore password.
     * @param keyStoreType The keystore type (i.e., JKS).
     * @param keyStore The keystore location.
     * @param keyStorePassword The keystore password.
     * @param x509Algorithm The X509 algorithm name to use for the
     *        trust and key manager factories.
     *
     * @throws IOException upon error.
     */
    public void init(String sslProtocol, String trustStoreType,
        String trustStore, String trustStorePassword, String keyStoreType,
        String keyStore, String keyStorePassword, String x509Algorithm)
        throws IOException;

    /**
     * Retrieves the SSLSocketFactory from the established underlying
     * SSLContext.
     *
     * @return An instance of javax.net.SSLSocketFactory.
     *
     * @throws IOException upon error.
     */
    public SSLSocketFactory getSocketFactory()
        throws IOException;
}
